indexing

	description:

		"External URI resolver for the http protocol"

	library: "eposix"
	author: "Berend de Boer <berend@pobox.com>"
	copyright: "Copyright (c) 2004, Berend de Boer"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2005/03/03 $"
	revision: "$Revision: #3 $"


class

	EPX_HTTP_URI_RESOLVER


inherit

	XM_URI_RESOLVER


feature -- Operation(s)

	scheme: STRING is "http"

	resolve (a_uri: UT_URI) is
			-- Resolve URI to stream.
		local
			path: STRING
			stream: KI_CHARACTER_INPUT_STREAM
		do
			if a_uri.is_server_authority then
				if a_uri.host = Void then
					a_uri.parse_authority (80)
				end
				create client.make_from_port (a_uri.host, a_uri.port)
				create path.make_from_string (a_uri.path)
				if a_uri.query /= Void then
					path.append_character ('?')
					path.append_string (a_uri.query)
				end
				if a_uri.fragment /= Void then
					path.append_character ('#')
					path.append_string (a_uri.fragment)
				end
				client.get (a_uri.path)
				if client.response_code = 200 then
					client.read_response
					-- Better would be just to read the header, and have
					-- `last_stream' deliver the actual body.
					-- Set name of stream to the URI.
					stream := last_stream
					if stream /= Void then
						stream.name.wipe_out
						stream.name.append_string (a_uri.full_uri)
					end
				end
			else
				set_local_error ("Host name needed with http protocol")
			end
		end


feature -- Result

	last_stream: KI_CHARACTER_INPUT_STREAM is
			-- Matching stream
		do
			if client /= Void and then client.body /= Void then
				Result := client.body.stream
			end
		end

	has_error: BOOLEAN is
			-- Did the last resolution attempt succeed?
		do
			Result := has_local_error or else not client.is_response_ok
		end

	last_error: STRING is
		do
			Result := client.response_phrase
		end


feature {NONE} -- Implementation

	client: EPX_HTTP_11_CLIENT
			-- HTTP protocol

	has_local_error: BOOLEAN

	last_local_error: STRING

	set_local_error (an_error_string: STRING) is
			-- Set a local error.
		require
			error_string_not_void: an_error_string /= Void
		do
			has_local_error := True
			last_local_error := an_error_string
		ensure
			error_set: has_local_error and then STRING_.same_string (an_error_string, last_local_error)
		end

end
